/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * This library is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library. If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined (__LIBICAL_GLIB_H_INSIDE__) && !defined (LIBICAL_GLIB_COMPILATION)
#error "Only <libical-glib/libical-glib.h> can be included directly."
#endif

#ifndef I_CAL_PARAMETER_H
#define I_CAL_PARAMETER_H

#include <libical-glib/i-cal-object.h>
#include <libical-glib/i-cal-derived-parameter.h>

G_BEGIN_DECLS

#define I_CAL_PARAMETER_TYPE \
        (i_cal_parameter_get_type ())
#define I_CAL_PARAMETER(obj) \
        (G_TYPE_CHECK_INSTANCE_CAST \
        ((obj), I_CAL_PARAMETER_TYPE, ICalParameter))
#define I_CAL_PARAMETER_CLASS(klass) \
        (G_TYPE_CHECK_CLASS_CAST \
        ((klass), I_CAL_PARAMETER_TYPE, ICalParameterClass))
#define I_CAL_IS_PARAMETER(obj) \
        (G_TYPE_CHECK_INSTANCE_TYPE \
        ((obj), I_CAL_PARAMETER_TYPE))
#define I_CAL_IS_PARAMETER_CLASS(klass) \
        (G_TYPE_CHECK_CLASS_TYPE \
        ((klass), I_CAL_PARAMETER_TYPE))

/**
 * ICalParameter:
 *
 * This is the ICalParameter instance.
 */
typedef struct _ICalParameter ICalParameter;

/**
 * ICalParameterClass:
 *
 * This is the ICalParameter class.
 */
typedef struct _ICalParameterClass ICalParameterClass;

struct _ICalParameter {
	/*< private >*/
	ICalObject parent;
};

struct _ICalParameterClass {
	/*< private >*/
	ICalObjectClass parent;
};

ICalParameter *	i_cal_parameter_new_full	(icalparameter *native,
						 GObject *owner);
GType 		i_cal_parameter_get_type	(void);
ICalParameter *	i_cal_parameter_new		(ICalParameterKind v);
ICalParameter *	i_cal_parameter_new_clone	(ICalParameter *p);
ICalParameter *	i_cal_parameter_new_from_string	(const gchar *value);
ICalParameter *	i_cal_parameter_new_from_value_string
						(ICalParameterKind kind,
						 const gchar *value);
void		i_cal_parameter_free		(ICalParameter *parameter);
gchar *		i_cal_parameter_as_ical_string_r
						(ICalParameter *parameter);
ICalParameterKind 
		i_cal_parameter_isa		(ICalParameter *parameter);
gint 		i_cal_parameter_isa_parameter	(ICalParameter *param);
void		i_cal_parameter_set_xname	(ICalParameter *param,
						 const gchar *v);
const gchar *	i_cal_parameter_get_xname	(ICalParameter *param);
void		i_cal_parameter_set_xvalue	(ICalParameter *param,
						 const gchar *v);
const gchar *	i_cal_parameter_get_xvalue	(ICalParameter *param);
void		i_cal_parameter_set_iana_name	(ICalParameter *param,
						 const gchar *v);
const gchar *	i_cal_parameter_get_iana_name	(ICalParameter *param);
void		i_cal_parameter_set_iana_value	(ICalParameter *param,
						 const gchar *v);
const gchar *	i_cal_parameter_get_iana_value	(ICalParameter *param);
gint 		i_cal_parameter_has_same_name	(ICalParameter *param1,
						 ICalParameter *param2);
const gchar *	i_cal_parameter_kind_to_string	(ICalParameterKind kind);
ICalParameterKind 
		i_cal_parameter_string_to_kind	(const gchar *string);

G_END_DECLS

#endif /* I_CAL_PARAMETER_H */